KeystoneRollCall = LibStub('AceAddon-3.0'):NewAddon('KeystoneRollCall', 'AceEvent-3.0', 'AceConsole-3.0','AceTimer-3.0');
_G.KeystoneRollCall = KeystoneRollCall;
local AceGUI = LibStub('AceGUI-3.0');	
local ScrollingTable = LibStub('ScrollingTable');

local _,bag,bagSize,i,msg,dungeonId,key,val
local sortFields = {{"name",true}}
local MSG_PREFIX = "KRC-MSG"
local events = {}
local playerName
local playerRealm

function KeystoneRollCall:GetKeystoneIndex(characterName)
	for i, keystone in pairs(self.db.global.keystones) do
		if keystone.name == characterName then
			return i
		end
	end
	return -1
end
local CLASS_COLOR_CODES = {
	["Warrior"] = "|cFFC79C6E",
	["Monk"] = "|cFF558A84",
	["Death Knight"] = "|cFFC41F3B",
	["Shaman"] = "|cFF0070DE",
	["Priest"] = "|cFFFFFFFF",
	["Rogue"] = "|cFFFFF569",
	["Hunter"] = "|cFFABD473",
	["Paladin"] = "|cFFF58CBA",
	["Warlock"] = "|cFF9482C9",
	["Mage"] = "|cFF69CCF0",
	["Druid"] = "|cFFFF7D0A",
	["Demon Hunter"] = "|cFFA330C9"
}

local function KeystoneToString(keystone)
	if keystone.level == 0 then
		return "No keystone found for " .. keystone.name
	end
	return "+" .. keystone.level .. " ("  ..  C_ChallengeMode.GetMapInfo(keystone.dungeon) .. ")  " ..  keystone.name
end

function KeystoneRollCall:UpdatePlayerKeystone()
	if not playerName then
		return
	end
	
	
	C_ChallengeMode.RequestMapInfo();
	C_ChallengeMode.RequestRewards();
	local mapTable = C_ChallengeMode.GetMapTable();
	local bestLevel = 0;
	local bestDungeon;
	for i, dungeonId in pairs(mapTable) do
		local _, weeklyBestTime, weeklyBestLevel = C_ChallengeMode.GetMapPlayerStats(dungeonId);

		if weeklyBestLevel and weeklyBestLevel > bestLevel then
			bestLevel = weeklyBestLevel;
			bestDungeon = dungeonId;
		end
	end
	
	local keystone = {}
	keystone.name = playerName;
	keystone.class = UnitClass("player");
	keystone.isPlayers = true;
	keystone.bestLevel = bestLevel
	keystone.bestDungeon = bestDungeon
	keystone.faction = UnitFactionGroup("player") == "Alliance"
	keystone.dungeon = 0;
	keystone.level = 0;
	
	
	local keystoneIndex = self:GetKeystoneIndex(playerName)
	local newKey = false;
	for bag = 0, NUM_BAG_SLOTS do
		bagSize = GetContainerNumSlots(bag);
		if bagSize > 0 then
			for i = 1, bagSize do
				if (GetContainerItemID(bag, i) == 138019) then
					local keyParts = { strsplit(':[', GetContainerItemLink(bag, i)) }
					
					keystone.dungeon = tonumber(keyParts[2]);
					keystone.level = tonumber(keyParts[3]);
					keystone.affix1 = tonumber(keyParts[4]);
					keystone.affix2 = tonumber(keyParts[5]);
					keystone.affix3 = tonumber(keyParts[6]);
				end
			end
		end
	end
	if keystoneIndex and  keystoneIndex >0 then
		local oldKeystone = self.db.global.keystones[keystoneIndex]
		for key,val in pairs(keystone) do
			if oldKeystone[key] ~= val then
				newKey = true;
				break
			end	
		end
		self.db.global.keystones[keystoneIndex] = keystone
	else
		tinsert(self.db.global.keystones,keystone)
	end
	return newKey
end

function KeystoneRollCall:SendKeystone(requestSync,keystoneOwner,channel,target)
	local keystoneIndex = self:GetKeystoneIndex(keystoneOwner)
	if keystoneIndex == -1 then
		msg = table.concat({requestSync, keystoneOwner, -1.-1.-1,-1,-1,-1,-1,-1,-1}, ",") 
	else
		local keystone = self.db.global.keystones[keystoneIndex]
		msg = table.concat({requestSync, keystone.name, keystone.class,keystone.faction and 1 or 0,keystone.dungeon,keystone.level,keystone.affix1 or 0,keystone.affix2 or 0,keystone.affix3 or 0,keystone.bestLevel,keystone.bestDungeon}, ",") 
	end
	SendAddonMessage(MSG_PREFIX, msg,channel, channel == "whisper" and target or nil)
end

function KeystoneRollCall:ReportAvailableKeys(channel,target)

	self:DoSort()
	SendChatMessage("Keystone Roll-Call (Personal Keys):",channel, "Common",target)
	for i, keystone in pairs(self.db.global.keystones) do
		if tonumber(keystone.dungeon) > 0 and keystone.isPlayers then
			SendChatMessage("   " .. KeystoneToString(keystone),channel ,  "Common",target)
		end
	end
end

function KeystoneRollCall:ReportPartyKeys(channel,target)
	self:DoSort()
	SendChatMessage("Keystone Roll-Call (Party/Raid Keys):",channel, "Common",target)
	
	for i = 1,40 do
		local name,realm= UnitName("raid"..i)
		if name then
			realm = realm or playerRealm
			
			local keystoneIndex = self:GetKeystoneIndex(name.."-"..realm)
		
			if keystoneIndex ~= -1 then
				SendChatMessage("   " .. KeystoneToString(self.db.global.keystones[keystoneIndex]),channel ,  "Common",target)
			end
		end
	end
end

function KeystoneRollCall:LocalReportAvailableKeys()
	print("Keystone Roll-Call (" .. #self.db.global.keystones .. ")")
	for i, keystone in ipairs(self.db.global.keystones) do
		if tonumber(keystone.dungeon) > 0 and keystone.isPlayers then
			msg = "   " .. (keystone.name or "") .. "  -  |cFFFFFF00" .. C_ChallengeMode.GetMapInfo(keystone.dungeon) .. "|r +" 
			if keystone.level >= 10 then
				print(msg.. "|cFFFF0000" ..  keystone.level .."|r")
			elseif keystone.level >= 7 then
				print(msg.. "|cFFFFFF00" ..  keystone.level .."|r")
			elseif keystone.level >= 4 then
				print(msg.. "|cFF00FF00" ..  keystone.level .."|r")
			else
				print(msg..  keystone.level )
			end
		end
	end
end

function KeystoneRollCall:PurgeExpiredKeys() 
	local keystoneIndex = self:GetKeystoneIndex(playerName)
	if keystoneIndex == -1 then
		return
	end
	
	local affix1 = self.db.global.keystones[keystoneIndex].affix1
	if not affix1 or affix1 == 0 then
		return
	end
	
	for i = #self.db.global.keystones, 1, -1 do
		local oldAffix1 = self.db.global.keystones[i].affix1;
		if oldAffix1 and oldAffix1 > 0 and oldAffix1 ~= affix1 then
			tremove(self.db.global.keystones,i)
		end
	end
end

function KeystoneRollCall:OnInitialize() 
	self.db = LibStub('AceDB-3.0'):New('KRCDB', {global={keystones={}}}, 'Default')
	local count = #self.db.global.keystones;
	--for i =1, count do
	--	tremove(self.db.global.keystones,1);
	--end
	
	if not self.db.global.minimapIcon then
		self.db.global.minimapIcon = {}
	end
		
	self:RegisterChatCommand("krc", "SlashCommand")
	local dataobj = LibStub:GetLibrary('LibDataBroker-1.1'):NewDataObject("Keystone Roll-Call", {
		type = 'launcher',
		text = "Keystone Roll-Call",
		label = "Keystone Roll-Call",
		icon = 'Interface\\Icons\\Ability_hunter_beastcall',
	})
	
	dataobj.OnClick = function(self,button,down) 
		if not KeystoneRollCall.frame or not KeystoneRollCall.frame:IsVisible() then
			KeystoneRollCall:Show()
		end
	end
	dataobj.OnTooltipShow = function(tooltip)
		KeystoneRollCall:DoSort()
		tooltip:AddLine("Keystone Roll-Call")
		tooltip:AddLine(" ")
		
		
		for i, keystone in ipairs(self.db.global.keystones) do
			if tonumber(keystone.dungeon) > 0 and keystone.isPlayers then
				local levelColor = keystone.level >= 10 and "|cFFFF0000" or keystone.level >= 7  and "|cFFFFFF00"  or keystone.level >= 4  and "|cFF00FF00" or "|cFFFFFFFF" 
				tooltip:AddDoubleLine("+".. levelColor..keystone.level .. "|r " ..C_ChallengeMode.GetMapInfo(keystone.dungeon),CLASS_COLOR_CODES[keystone.class] .. keystone.name .."|r");
			end
		end
	end
	
	self.icon = LibStub("LibDBIcon-1.0", true)
	self.icon:Register("Keystone Roll-Call", dataobj, self.db.global.minimapIcon)
end


function KeystoneRollCall:SlashCommand(args)
	if not KeystoneRollCall.frame or not KeystoneRollCall.frame:IsVisible() then
		KeystoneRollCall:Show()
	end
end

function events:GROUP_JOINED() 
	KeystoneRollCall:SendKeystone(1,playerName,"RAID")
end

function events:PLAYER_LOGIN()
	local name, realm = UnitFullName("player")
	playerName = name .."-" .. realm
	playerRealm = realm
	KeystoneRollCall:UpdatePlayerKeystone()
	KeystoneRollCall:PurgeExpiredKeys()
	RegisterAddonMessagePrefix(MSG_PREFIX)
	KeystoneRollCall:SendKeystone(1,playerName,"GUILD")
end

function events:BAG_UPDATE()
	if KeystoneRollCall:UpdatePlayerKeystone() then
		KeystoneRollCall:PurgeExpiredKeys()
		KeystoneRollCall:SendKeystone(0,playerName,"GUILD")
	end
end

function events:CHAT_MSG_ADDON(prefix,msg,channel,sender) 
	if prefix == MSG_PREFIX then
		if sender == playerName then
			--print("IGNORED:",msg)
			return
		end

		local msgParts =  {strsplit(",",msg)};
		if msgParts[1] == "1" then
			for i, keystone in ipairs(KeystoneRollCall.db.global.keystones) do
				if keystone.isPlayers then
					KeystoneRollCall:SendKeystone(0,keystone.name,channel,sender)
				end
			end
			
		end

		
		local keystone = {}
		keystone.name = msgParts[2]
		keystone.class = msgParts[3]
		keystone.faction = msgParts[4] == "1"
		keystone.dungeon = tonumber(msgParts[5])
		keystone.level = tonumber(msgParts[6])
		keystone.affix1 = tonumber(msgParts[7])
		keystone.affix2 = tonumber(msgParts[8])
		keystone.affix3 = tonumber(msgParts[9])
		keystone.bestLevel = tonumber(msgParts[10])
		keystone.bestDungeon = tonumber(msgParts[11])
		keystone.isPlayers = false;
		
		local keystoneIndex = KeystoneRollCall:GetKeystoneIndex(msgParts[2])
		
		
		if keystoneIndex == -1 then
			if keystone.bestLevel > 0 then
				--Announce new best level.
			end
			if keystone.level > 0 then
				--Announce new keystone
			end 
		
			table.insert(KeystoneRollCall.db.global.keystones, keystone)
		else 
			local oldKeystone = KeystoneRollCall.db.global.keystones[keystoneIndex]
			if oldKeystone.isPlayers then
				keystone.isPlayers = true
			end
			
			if keystone.bestLevel > 0 and keystone.bestLevel ~= oldKeystone.bestLevel then
				--Announce new best level.
			end
			
			if keystone.level > 0 and ( keystone.level ~= oldKeystone.level or keystone.dungeon ~= oldKeystone.dungeon)  then
				--Announce new keystone
			end 
			KeystoneRollCall.db.global.keystones[keystoneIndex] = keystone
		end
	end
end



local function EventFunc(self, event, ...)
	events[event](event, ...)
end
local eventHandler = CreateFrame("Frame", nil, WorldFrame)
eventHandler:SetScript("OnEvent", EventFunc)
for eventname in pairs(events) do eventHandler:RegisterEvent(eventname) end	





local function CompareKeystones(field,asc,ka,kb)
	if sortFields[1][1] == field then
		sortFields[1][2] = asc
	else
		for i = 2, #sortFields do
			if sortFields[i] and sortFields[i][1] == field then
				tremove(sortFields,i)
			end
		end
		tinsert(sortFields,1,{field,true})
	end


	for i = 1, #sortFields do
		
		local av
		local bv
		if sortFields[i][1] == "dungeon" then
			av = C_ChallengeMode.GetMapInfo(ka.dungeon) or "ZZZ"
			bv = C_ChallengeMode.GetMapInfo(kb.dungeon) or "ZZZ"
		elseif sortFields[i][1] == "realm" then
			local nameParts =  {strsplit("-",ka.name)}
			av = nameParts[2]
			nameParts =  {strsplit("-",kb.name)}
			bv = nameParts[2]
		else
			av =  ka[sortFields[i][1]]
			bv =  kb[sortFields[i][1]]
		end
		if av < bv then
			return sortFields[i][2];
		elseif av > bv then
			return not sortFields[i][2];
		end
	end
	return 

end


function KeystoneRollCall:DoSort()
	table.sort(self.db.global.keystones, function(ka,kb) 
		return CompareKeystones(sortFields[1][1],sortFields[1][2],ka,kb)
	end);
end

function KeystoneRollCall:PopulatePersonal()
	for i, keystone in ipairs(KeystoneRollCall.db.global.keystones) do
		if keystone.isPlayers then
			local nameParts =  {strsplit("-",keystone.name)};
			local levelColor = keystone.level >= 10 and "|cFFFF0000" or keystone.level >= 7  and "|cFFFFFF00"  or keystone.level >= 4  and "|cFF00FF00" or  "|cFFFFFFFF" 
			local realmColor = keystone.faction and "|cFF6666FF" or "|cFFFF0000"
			local bestLevelLine = ""
			if keystone.bestLevel and keystone.bestDungeon then
				bestLevelLine = ("+" .. (keystone.bestLevel and keystone.bestLevel>=10 and "|cFF00FF00" or "|cFFFF3333")..keystone.bestLevel .."|r (" .. C_ChallengeMode.GetMapInfo(keystone.bestDungeon) ..")")
			end

			tinsert(KeystoneRollCall.tableData, {
				i,
				realmColor .. nameParts[2].."|r",
				CLASS_COLOR_CODES[keystone.class]..nameParts[1].."|r",
				CLASS_COLOR_CODES[keystone.class]..keystone.class.."|r",
				C_ChallengeMode.GetMapInfo(keystone.dungeon),
				keystone.level == 0 and " " or (levelColor .. keystone.level .. "|r"),
				bestLevelLine
			});
		end
	end
	
	KeystoneRollCall.scrollTable:SortData();
end


local function DrawPersonalTab(container)	
	local cols = {{
			['name'] = '',
			['width'] = 1,
			['align'] = 'LEFT'
		},{
			['name'] = 'Realm',
			['width'] = 65,
			['align'] = 'LEFT',
			["comparesort"] = function (table, rowa, rowb, sortbycol)
				local asc = table.cols[sortbycol].sort == "asc"
				local ka = KeystoneRollCall.db.global.keystones[KeystoneRollCall.tableData[rowa][1]]
				local kb = KeystoneRollCall.db.global.keystones[KeystoneRollCall.tableData[rowb][1]]
				return CompareKeystones("realm", asc, ka, kb);
			end,
			['sort'] = "asc"
		},{
			['name'] = 'Character',
			['width'] = 85,
			['align'] = 'LEFT', 
			["comparesort"] = function (table, rowa, rowb, sortbycol)
				local asc = table.cols[sortbycol].sort == "asc"
				local ka = KeystoneRollCall.db.global.keystones[KeystoneRollCall.tableData[rowa][1]]
				local kb = KeystoneRollCall.db.global.keystones[KeystoneRollCall.tableData[rowb][1]]
				return CompareKeystones("name", asc, ka, kb);
			end,
			['sort'] = "asc"
		},{
			['name'] = 'Class',
			['width'] = 95,
			['align'] = 'LEFT', 
			["comparesort"] = function (table, rowa, rowb, sortbycol)
				local asc = table.cols[sortbycol].sort == "asc"
				local ka = KeystoneRollCall.db.global.keystones[KeystoneRollCall.tableData[rowa][1]]
				local kb = KeystoneRollCall.db.global.keystones[KeystoneRollCall.tableData[rowb][1]]
				return CompareKeystones("class", asc, ka, kb);
			end,
			['sort'] = "asc"
		},{
			['name'] = 'Dungeon',
			['width'] = 160,
			['align'] = 'LEFT',
			["comparesort"] = function (table, rowa, rowb, sortbycol)
				local asc = table.cols[sortbycol].sort == "asc"
				local ka = KeystoneRollCall.db.global.keystones[KeystoneRollCall.tableData[rowa][1]]
				local kb = KeystoneRollCall.db.global.keystones[KeystoneRollCall.tableData[rowb][1]]
				return CompareKeystones("dungeon", asc, ka, kb);
			end,
			['sort'] = "asc"
		},{
			['name'] = 'Level',
			['width'] = 40,
			['align'] = 'LEFT',
			["comparesort"] = function (table, rowa, rowb, sortbycol)
				local asc = table.cols[sortbycol].sort == "asc"
				local ka = KeystoneRollCall.db.global.keystones[KeystoneRollCall.tableData[rowa][1]]
				local kb = KeystoneRollCall.db.global.keystones[KeystoneRollCall.tableData[rowb][1]]
				return CompareKeystones("level", asc, ka, kb);
			end,
			['sort'] = "asc"
		},{
			['name'] = "Best Run (This Week)",
			['width'] = 180,
			['align'] = 'LEFT',
		}
	}
	KeystoneRollCall.tableData = {};
	KeystoneRollCall.scrollTable = ScrollingTable:CreateST(cols,21);
	KeystoneRollCall.scrollTable:SetData(KeystoneRollCall.tableData, true);

	KeystoneRollCall.scrollTable:RegisterEvents({
		['OnClick'] = function(rowFrame, cellFrame, data, cols, row, realrow, column, scrollingTable, btn,...)
			if KeystoneRollCall.tableData and KeystoneRollCall.tableData[realrow] and KeystoneRollCall.tableData[realrow][1] ~= -1 then
				local keystone = KeystoneRollCall.db.global.keystones[realrow];
				if btn == "RightButton" then
					if UnitName("target") then
						SendChatMessage(KeystoneToString(keystone),"WHISPER",  "Common",UnitName("target"));
					end
				elseif IsInGroup() then
					SendChatMessage(KeystoneToString(keystone),"RAID" ,  "Common")
				end
			
			end
		end,	
		['OnDoubleClick'] = function(rowFrame, cellFrame, data, cols, row, realrow, column, scrollingTable, btn,...)
			if KeystoneRollCall.tableData and KeystoneRollCall.tableData[realrow] and  KeystoneRollCall.tableData[realrow][1] ~= -1 then
				local keystone = KeystoneRollCall.db.global.keystones[realrow];
				SendChatMessage(KeystoneToString(keystone),"GUILD" ,  "Common")
			end
		end,	
		['OnEnter'] = function(rowFrame, cellFrame, data, cols, row, realrow, column, scrollingTable,...)
			if KeystoneRollCall.tableData and KeystoneRollCall.tableData[realrow] and  KeystoneRollCall.tableData[realrow][1] ~= -1 then
				KeystoneRollCall.frame:SetStatusText("Left Click: Send to Party   -   Double Click: Send to Guild   -   Right Click: Send to Target")
			end
		end,
		['OnLeave'] = function(rowFrame, cellFrame, data, cols, row, realrow, column, scrollingTable,...)
			KeystoneRollCall.frame:SetStatusText("")
		end,
	});
	
	
	local tableWrapper = AceGUI:Create('lib-st'):WrapST(KeystoneRollCall.scrollTable);

	tableWrapper.head_offset = 15;
	container:AddChild(tableWrapper);
	
	local spacer = AceGUI:Create('Heading');
	spacer:SetFullWidth(true);
	spacer:SetText(' ');

	container:AddChild(spacer);
	
	
	local broadcastPartyButton = AceGUI:Create('Button');
	broadcastPartyButton:SetWidth(200);
	broadcastPartyButton:SetText('Broadcast All to Party/Raid');

	broadcastPartyButton:SetCallback('OnClick', function()
		KeystoneRollCall:ReportAvailableKeys("RAID");
	end);
	container:AddChild(broadcastPartyButton);
	
	
	local broadcastGuildButton = AceGUI:Create('Button');
	broadcastGuildButton:SetWidth(200);
	broadcastGuildButton:SetText('Broadcast All to Guild');

	broadcastGuildButton:SetCallback('OnClick', function()
		KeystoneRollCall:ReportAvailableKeys("GUILD");
	end);
	container:AddChild(broadcastGuildButton);
	
	local whisperTargetButton = AceGUI:Create('Button');
	whisperTargetButton:SetWidth(200);
	whisperTargetButton:SetText('Whisper All to Target');

	whisperTargetButton:SetCallback('OnClick', function()
		if UnitName("target") then
			KeystoneRollCall:ReportAvailableKeys("WHISPER",UnitName("target"));
		end
	end);
	container:AddChild(whisperTargetButton);
end


function KeystoneRollCall:PopulateParty()
	for i = 1,40 do
		if UnitExists("raid"..i) then
			local name, realm = UnitName("raid"..i) or ""
			realm = realm or playerRealm
			local fullName = name .."-"..realm
			local keystoneIndex = KeystoneRollCall:GetKeystoneIndex(fullName)
			local class = UnitClass("raid"..i) or "Warrior"
			local role = UnitGroupRolesAssigned("raid"..i) or ""
			role = role == "DAMAGER" and "|cFF6666FFDPS|r" or role == "HEALER" and "|cFFFFFF00Healer|r" or role == "TANK" and"|cFFFF0000Tank|r" or ""
			if keystoneIndex == -1 then
				KeystoneRollCall.dummyKeystones["raid"..i] = {['name'] =fullName,['class'] = class,dungeon = -1,level = -1}
				tinsert(KeystoneRollCall.tableData, {
					keystoneIndex,
					"raid"..i,
					role,
					CLASS_COLOR_CODES[class]..name.."|r",
					CLASS_COLOR_CODES[class]..class.."|r",
					"",
					"",
					""
				});
			else
				local keystone = KeystoneRollCall.db.global.keystones[keystoneIndex]
				local levelColor = keystone.level >= 10 and "|cFFFF0000" or keystone.level >= 7  and "|cFFFFFF00"  or keystone.level >= 4  and "|cFF00FF00" or  keystone.level > 0  and "|cFFFFFFFF" or ""
				local bestLevelLine = ""
				if keystone.bestLevel and keystone.bestDungeon then
					bestLevelLine = ("+" .. (keystone.bestLevel and keystone.bestLevel>=10 and "|cFF00FF00" or "|cFFFF3333")..keystone.bestLevel .."|r (" .. C_ChallengeMode.GetMapInfo(keystone.bestDungeon) ..")")
				end

				tinsert(KeystoneRollCall.tableData, {
					keystoneIndex,
					"raid"..i,
					role,
					CLASS_COLOR_CODES[class]..name.."|r",
					CLASS_COLOR_CODES[class]..class.."|r",
					C_ChallengeMode.GetMapInfo(keystone.dungeon),
					keystone.level == 0 and " " or (levelColor .. keystone.level .. "|r"),
					bestLevelLine
				});
			end
		end
	end
	
	KeystoneRollCall.scrollTable:SortData();
end


local function DrawPartyTab(container)
	local cols = {{
			['name'] = '',
			['width'] = 1,
			['align'] = 'LEFT'
		},{
			['name'] = '',
			['width'] = 1,
			['align'] = 'LEFT'
		},{
			['name'] = 'Role',
			['width'] = 65,
			['align'] = 'LEFT'
		},{
			['name'] = 'Character',
			['width'] = 85,
			['align'] = 'LEFT', 
			["comparesort"] = function (table, rowa, rowb, sortbycol)
				local asc = table.cols[sortbycol].sort == "asc"
				local ka,kb
				if KeystoneRollCall.tableData[rowa][1] == -1 then
					ka = KeystoneRollCall.dummyKeystones[KeystoneRollCall.tableData[rowa][2]]
				else
					ka = KeystoneRollCall.db.global.keystones[KeystoneRollCall.tableData[rowa][1]]
				end
				if KeystoneRollCall.tableData[rowb][1] == -1 then
					kb = KeystoneRollCall.dummyKeystones[KeystoneRollCall.tableData[rowb][2]]
				else
					kb = KeystoneRollCall.db.global.keystones[KeystoneRollCall.tableData[rowb][1]]
				end
				return CompareKeystones("name", asc, ka, kb);
			end,
			['sort'] = "asc"
		},{
			['name'] = 'Class',
			['width'] = 95,
			['align'] = 'LEFT', 
			["comparesort"] = function (table, rowa, rowb, sortbycol)
				local asc = table.cols[sortbycol].sort == "asc"
				local ka,kb
				if KeystoneRollCall.tableData[rowa][1] == -1 then
					ka = KeystoneRollCall.dummyKeystones[KeystoneRollCall.tableData[rowa][2]]
				else
					ka = KeystoneRollCall.db.global.keystones[KeystoneRollCall.tableData[rowa][1]]
				end
				if KeystoneRollCall.tableData[rowb][1] == -1 then
					kb = KeystoneRollCall.dummyKeystones[KeystoneRollCall.tableData[rowb][2]]
				else
					kb = KeystoneRollCall.db.global.keystones[KeystoneRollCall.tableData[rowb][1]]
				end
				return CompareKeystones("class", asc, ka, kb);
			end,
			['sort'] = "asc"
		},{
			['name'] = 'Dungeon',
			['width'] = 160,
			['align'] = 'LEFT',
			["comparesort"] = function (table, rowa, rowb, sortbycol)
				local asc = table.cols[sortbycol].sort == "asc"
				local ka,kb
				if KeystoneRollCall.tableData[rowa][1] == -1 then
					ka = KeystoneRollCall.dummyKeystones[KeystoneRollCall.tableData[rowa][2]]
				else
					ka = KeystoneRollCall.db.global.keystones[KeystoneRollCall.tableData[rowa][1]]
				end
				if KeystoneRollCall.tableData[rowb][1] == -1 then
					kb = KeystoneRollCall.dummyKeystones[KeystoneRollCall.tableData[rowb][2]]
				else
					kb = KeystoneRollCall.db.global.keystones[KeystoneRollCall.tableData[rowb][1]]
				end
				return CompareKeystones("dungeon", asc, ka, kb);
			end,
			['sort'] = "asc"
		},{
			['name'] = 'Level',
			['width'] = 40,
			['align'] = 'LEFT',
			["comparesort"] = function (table, rowa, rowb, sortbycol)
				local asc = table.cols[sortbycol].sort == "asc"
				local ka,kb
				if KeystoneRollCall.tableData[rowa][1] == -1 then
					ka = KeystoneRollCall.dummyKeystones[KeystoneRollCall.tableData[rowa][2]]
				else
					ka = KeystoneRollCall.db.global.keystones[KeystoneRollCall.tableData[rowa][1]]
				end
				if KeystoneRollCall.tableData[rowb][1] == -1 then
					kb = KeystoneRollCall.dummyKeystones[KeystoneRollCall.tableData[rowb][2]]
				else
					kb = KeystoneRollCall.db.global.keystones[KeystoneRollCall.tableData[rowb][1]]
				end
				return CompareKeystones("level", asc, ka, kb);
			end,
			['sort'] = "asc"
		},{
			['name'] = "Best Run (This Week)",
			['width'] = 180,
			['align'] = 'LEFT',
		}
	}
	KeystoneRollCall.tableData = {};
	KeystoneRollCall.dummyKeystones = {};
	KeystoneRollCall.scrollTable = ScrollingTable:CreateST(cols,21);	
	KeystoneRollCall.scrollTable:SetData(KeystoneRollCall.tableData, true);

	KeystoneRollCall.scrollTable:RegisterEvents({
		['OnClick'] = function(rowFrame, cellFrame, data, cols, row, realrow, column, scrollingTable, btn,...)
			if not KeystoneRollCall.tableData or not KeystoneRollCall.tableData[realrow] then
				return
			end
			if KeystoneRollCall.tableData[realrow][1] == -1 then
				local keystone = KeystoneRollCall.dummyKeystones[KeystoneRollCall.tableData[realrow][2]]

				SendChatMessage("If you get the addon Keystone Roll-Call we can easily view each other's keystones, as well as keystones from across our guilds.","WHISPER",  "Common",keystone.name);
			elseif KeystoneRollCall.db.global.keystones[KeystoneRollCall.tableData[realrow][1]] then
				local keystone = KeystoneRollCall.db.global.keystones[KeystoneRollCall.tableData[realrow][1]];
				if btn == "RightButton" then
					if UnitName("target") then
						SendChatMessage(KeystoneToString(keystone),"WHISPER", "COMMON",  UnitName("target"));
					end
				elseif IsInGroup() then
					SendChatMessage(KeystoneToString(keystone),"RAID" ,  "Common")
				end
			
			end
		end,	
		['OnDoubleClick'] = function(rowFrame, cellFrame, data, cols, row, realrow, column, scrollingTable, btn,...)
			if KeystoneRollCall.tableData and KeystoneRollCall.tableData[realrow] and KeystoneRollCall.tableData[realrow][1] ~= -1  and KeystoneRollCall.db.global.keystones[KeystoneRollCall.tableData[realrow][1]] then
				local keystone = KeystoneRollCall.db.global.keystones[KeystoneRollCall.tableData[realrow][1]];
				SendChatMessage(KeystoneToString(keystone),"GUILD" ,  "Common")
			end
		end,	
		['OnEnter'] = function(rowFrame, cellFrame, data, cols, row, realrow, column, scrollingTable,...)
			if not realrow or not KeystoneRollCall.tableData[realrow] then
				return
				
			end
			if KeystoneRollCall.tableData[realrow][1] == -1 then
				KeystoneRollCall.frame:SetStatusText("Click: Suggest Addon")
			else
				KeystoneRollCall.frame:SetStatusText("Left Click: Send to Party   -   Double Click: Send to Guild   -   Right Click: Send to Target")
			end
		end,
		['OnLeave'] = function(rowFrame, cellFrame, data, cols, row, realrow, column, scrollingTable,...)
			KeystoneRollCall.frame:SetStatusText("")
		end,
	});
	
	
	local tableWrapper = AceGUI:Create('lib-st'):WrapST(KeystoneRollCall.scrollTable);

	tableWrapper.head_offset = 15;
	container:AddChild(tableWrapper);
	
	local spacer = AceGUI:Create('Heading');
	spacer:SetFullWidth(true);
	spacer:SetText(' ');

	container:AddChild(spacer);
	
	
	local broadcastPartyButton = AceGUI:Create('Button');
	broadcastPartyButton:SetWidth(200);
	broadcastPartyButton:SetText('Broadcast All to Party/Raid');

	broadcastPartyButton:SetCallback('OnClick', function()
		KeystoneRollCall:ReportPartyKeys("RAID");
	end);
	container:AddChild(broadcastPartyButton);
	
	
	local broadcastGuildButton = AceGUI:Create('Button');
	broadcastGuildButton:SetWidth(200);
	broadcastGuildButton:SetText('Broadcast All to Guild');

	broadcastGuildButton:SetCallback('OnClick', function()
		KeystoneRollCall:ReportPartyKeys("GUILD");
	end);
	container:AddChild(broadcastGuildButton);
	
	local whisperTargetButton = AceGUI:Create('Button');
	whisperTargetButton:SetWidth(200);
	whisperTargetButton:SetText('Whisper All to Target');

	whisperTargetButton:SetCallback('OnClick', function()
		if UnitName("target") then
			KeystoneRollCall:ReportPartyKeys("WHISPER",UnitName("target"));
		end
	end);
	container:AddChild(whisperTargetButton);
end

function KeystoneRollCall:PopulateGuildOnline()
	for i = 1, GetNumGuildMembers() do
		local fullName, rank, rankIndex, level, class, zone, note, officernote, online, status, classFileName, achievementPoints, achievementRank, isMobile, canSoR, reputation = GetGuildRosterInfo(i)
		if online then
			local nameParts =  {strsplit("-",fullName)};
			local keystoneIndex = KeystoneRollCall:GetKeystoneIndex(fullName)
			if keystoneIndex == -1 then
				KeystoneRollCall.dummyKeystones[fullName] = {['name'] =fullName,['class'] = class,dungeon = -1,level = -1}
				tinsert(KeystoneRollCall.tableData, {
					keystoneIndex,
					fullName,
					CLASS_COLOR_CODES[class]..nameParts[1].."|r",
					CLASS_COLOR_CODES[class]..class.."|r",
					"",
					"",
					""
				});
			else
				local keystone = KeystoneRollCall.db.global.keystones[keystoneIndex]
				local levelColor = keystone.level >= 10 and "|cFFFF0000" or keystone.level >= 7  and "|cFFFFFF00"  or keystone.level >= 4  and "|cFF00FF00" or  keystone.level > 0  and "|cFFFFFFFF" or ""
				local bestLevelLine = ""
				if keystone.bestLevel and keystone.bestDungeon then
					bestLevelLine = ("+" .. (keystone.bestLevel and keystone.bestLevel>=10 and "|cFF00FF00" or "|cFFFF3333")..keystone.bestLevel .."|r (" .. C_ChallengeMode.GetMapInfo(keystone.bestDungeon) ..")")
				end

				tinsert(KeystoneRollCall.tableData, {
					keystoneIndex,
					fullName,
					CLASS_COLOR_CODES[class]..nameParts[1].."|r",
					CLASS_COLOR_CODES[class]..class.."|r",
					C_ChallengeMode.GetMapInfo(keystone.dungeon),
					keystone.level == 0 and " " or (levelColor .. keystone.level .. "|r"),
					bestLevelLine
				});
			end
		
		end
	end
	
	KeystoneRollCall.scrollTable:SortData();
end

local function DrawGuildOnlineTab(container)
	local cols = {{
			['name'] = '',
			['width'] = 1,
			['align'] = 'LEFT'
		},{
			['name'] = '',
			['width'] = 1,
			['align'] = 'LEFT'
		},{
			['name'] = 'Character',
			['width'] = 90,
			['align'] = 'LEFT', 
			["comparesort"] = function (table, rowa, rowb, sortbycol)
				local asc = table.cols[sortbycol].sort == "asc"
				local ka,kb
				if KeystoneRollCall.tableData[rowa][1] == -1 then
					ka = KeystoneRollCall.dummyKeystones[KeystoneRollCall.tableData[rowa][2]]
				else
					ka = KeystoneRollCall.db.global.keystones[KeystoneRollCall.tableData[rowa][1]]
				end
				if KeystoneRollCall.tableData[rowb][1] == -1 then
					kb = KeystoneRollCall.dummyKeystones[KeystoneRollCall.tableData[rowb][2]]
				else
					kb = KeystoneRollCall.db.global.keystones[KeystoneRollCall.tableData[rowb][1]]
				end
				return CompareKeystones("name", asc, ka, kb);
			end,
			['sort'] = "asc"
		},{
			['name'] = 'Class',
			['width'] = 105,
			['align'] = 'LEFT', 
			["comparesort"] = function (table, rowa, rowb, sortbycol)
				local asc = table.cols[sortbycol].sort == "asc"
				local ka,kb
				if KeystoneRollCall.tableData[rowa][1] == -1 then
					ka = KeystoneRollCall.dummyKeystones[KeystoneRollCall.tableData[rowa][2]]
				else
					ka = KeystoneRollCall.db.global.keystones[KeystoneRollCall.tableData[rowa][1]]
				end
				if KeystoneRollCall.tableData[rowb][1] == -1 then
					kb = KeystoneRollCall.dummyKeystones[KeystoneRollCall.tableData[rowb][2]]
				else
					kb = KeystoneRollCall.db.global.keystones[KeystoneRollCall.tableData[rowb][1]]
				end
				return CompareKeystones("class", asc, ka, kb);
			end,
			['sort'] = "asc"
		},{
			['name'] = 'Dungeon',
			['width'] = 170,
			['align'] = 'LEFT',
			["comparesort"] = function (table, rowa, rowb, sortbycol)
				local asc = table.cols[sortbycol].sort == "asc"
				local ka,kb
				if KeystoneRollCall.tableData[rowa][1] == -1 then
					ka = KeystoneRollCall.dummyKeystones[KeystoneRollCall.tableData[rowa][2]]
				else
					ka = KeystoneRollCall.db.global.keystones[KeystoneRollCall.tableData[rowa][1]]
				end
				if KeystoneRollCall.tableData[rowb][1] == -1 then
					kb = KeystoneRollCall.dummyKeystones[KeystoneRollCall.tableData[rowb][2]]
				else
					kb = KeystoneRollCall.db.global.keystones[KeystoneRollCall.tableData[rowb][1]]
				end
				return CompareKeystones("dungeon", asc, ka, kb);
			end,
			['sort'] = "asc"
		},{
			['name'] = 'Level',
			['width'] = 50,
			['align'] = 'LEFT',
			["comparesort"] = function (table, rowa, rowb, sortbycol)
				local asc = table.cols[sortbycol].sort == "asc"
				local ka,kb
				if KeystoneRollCall.tableData[rowa][1] == -1 then
					ka = KeystoneRollCall.dummyKeystones[KeystoneRollCall.tableData[rowa][2]]
				else
					ka = KeystoneRollCall.db.global.keystones[KeystoneRollCall.tableData[rowa][1]]
				end
				if KeystoneRollCall.tableData[rowb][1] == -1 then
					kb = KeystoneRollCall.dummyKeystones[KeystoneRollCall.tableData[rowb][2]]
				else
					kb = KeystoneRollCall.db.global.keystones[KeystoneRollCall.tableData[rowb][1]]
				end
				return CompareKeystones("level", asc, ka, kb);
			end,
			['sort'] = "asc"
		},{
			['name'] = "Best Run (This Week)",
			['width'] = 210,
			['align'] = 'LEFT',
		}
	}
	
	KeystoneRollCall.tableData = {};
	KeystoneRollCall.dummyKeystones = {};
	KeystoneRollCall.scrollTable = ScrollingTable:CreateST(cols,24);
	KeystoneRollCall.scrollTable:SetData(KeystoneRollCall.tableData, true);

	KeystoneRollCall.scrollTable:RegisterEvents({
		['OnClick'] = function(rowFrame, cellFrame, data, cols, row, realrow, column, scrollingTable, btn,...)
			if not KeystoneRollCall.tableData or not KeystoneRollCall.tableData[realrow] then
				return
			end
			if KeystoneRollCall.tableData[realrow][1] == -1 then
				local keystone = KeystoneRollCall.dummyKeystones[KeystoneRollCall.tableData[realrow][2]]

				SendChatMessage("If you get the addon Keystone Roll-Call we can easily view each other's keystones, as well as keystones from across our guilds.","WHISPER",  "Common",keystone.name);
			elseif KeystoneRollCall.db.global.keystones[KeystoneRollCall.tableData[realrow][1]] then
				local keystone = KeystoneRollCall.db.global.keystones[KeystoneRollCall.tableData[realrow][1]];
				if btn == "RightButton" then
					if UnitName("target") then
						SendChatMessage(KeystoneToString(keystone),"WHISPER", "COMMON",  UnitName("target"));
					end
				elseif IsInGroup() then
					SendChatMessage(KeystoneToString(keystone),"RAID" ,  "Common")
				end
			
			end
		end,	
		['OnDoubleClick'] = function(rowFrame, cellFrame, data, cols, row, realrow, column, scrollingTable, btn,...)
			if KeystoneRollCall.tableData and KeystoneRollCall.tableData[realrow] and KeystoneRollCall.tableData[realrow][1] ~= -1  and KeystoneRollCall.db.global.keystones[KeystoneRollCall.tableData[realrow][1]] then
				local keystone = KeystoneRollCall.db.global.keystones[KeystoneRollCall.tableData[realrow][1]];
				SendChatMessage(KeystoneToString(keystone),"GUILD" ,  "Common")
			end
		end,	
		['OnEnter'] = function(rowFrame, cellFrame, data, cols, row, realrow, column, scrollingTable,...)
			if not realrow or not KeystoneRollCall.tableData[realrow] then
				return
				
			end
			if KeystoneRollCall.tableData[realrow][1] == -1 then
				KeystoneRollCall.frame:SetStatusText("Click: Suggest Addon")
			else
				KeystoneRollCall.frame:SetStatusText("Left Click: Send to Party   -   Double Click: Send to Guild   -   Right Click: Send to Target")
			end
		end,
		['OnLeave'] = function(rowFrame, cellFrame, data, cols, row, realrow, column, scrollingTable,...)
			KeystoneRollCall.frame:SetStatusText("")
		end,
	});
	
	local tableWrapper = AceGUI:Create('lib-st'):WrapST(KeystoneRollCall.scrollTable);
	tableWrapper.head_offset = 15;
	container:AddChild(tableWrapper);
end


function KeystoneRollCall:PopulateGuildAll()
	for i = 1, GetNumGuildMembers() do
		local fullName, rank, rankIndex, level, class, zone, note, officernote, online, status, classFileName, achievementPoints, achievementRank, isMobile, canSoR, reputation = GetGuildRosterInfo(i)
		local nameParts =  {strsplit("-",fullName)};
		local lastOnline = "Now"
		local timeOffline = 0;
		if not online then
			local yearsOffline, monthsOffline, daysOffline, hoursOffline = GetGuildRosterLastOnline(i);

			timeOffline = hoursOffline+daysOffline*24 +monthsOffline*30*24*24 +monthsOffline*12*30*24
			if yearsOffline > 1 then
				lastOnline  = yearsOffline .." Years"
			elseif yearsOffline > 0 then
				lastOnline  = "1 Year"
			elseif monthsOffline > 1 then
				lastOnline  = monthsOffline .." Months"
			elseif monthsOffline > 0 then
				lastOnline  = "1 Month"
			elseif daysOffline > 1 then
				lastOnline  = daysOffline .." Days"
			elseif daysOffline > 0 then
				lastOnline  = "1 Day"
			elseif hoursOffline > 1 then
				lastOnline  = hoursOffline .." Hours"
			elseif hoursOffline > 0 then
				lastOnline  = "1 Hour"
			else
				lastOnline  = "<1 Hour"
			end
		end
		local keystoneIndex = KeystoneRollCall:GetKeystoneIndex(fullName)
		if keystoneIndex == -1 then
			KeystoneRollCall.dummyKeystones[fullName] = {['name'] =fullName,['class'] = class,dungeon = -1,level = -1}
			tinsert(KeystoneRollCall.tableData, {
				keystoneIndex,
				fullName,
				"",
				online and "Online" or "",
				CLASS_COLOR_CODES[class]..nameParts[1].."|r",
				CLASS_COLOR_CODES[class]..class.."|r",
				"",
				"",
				""
			});
		else
			local keystone = KeystoneRollCall.db.global.keystones[keystoneIndex]
			local levelColor = keystone.level >= 10 and "|cFFFF0000" or keystone.level >= 7  and "|cFFFFFF00"  or keystone.level >= 4  and "|cFF00FF00" or  keystone.level > 0  and "|cFFFFFFFF" or ""
			local bestLevelLine = ""
			if keystone.bestLevel and keystone.bestDungeon then
				bestLevelLine = ("+" .. (keystone.bestLevel and keystone.bestLevel>=10 and "|cFF00FF00" or "|cFFFF3333")..keystone.bestLevel .."|r (" .. C_ChallengeMode.GetMapInfo(keystone.bestDungeon) ..")")
			end

			tinsert(KeystoneRollCall.tableData, {
				keystoneIndex,
				fullName,
				"",
				online and "Online" or "",
				CLASS_COLOR_CODES[class]..nameParts[1].."|r",
				CLASS_COLOR_CODES[class]..class.."|r",
				C_ChallengeMode.GetMapInfo(keystone.dungeon),
				keystone.level == 0 and " " or (levelColor .. keystone.level .. "|r"),
				bestLevelLine
			});
		end		
	end
	KeystoneRollCall.scrollTable:SortData();
end

local function DrawGuildAllTab(container)
	local cols = {{
			['name'] = '',
			['width'] = 1,
			['align'] = 'LEFT'
		},{
			['name'] = '',
			['width'] = 1,
			['align'] = 'LEFT'
		},{
			['name'] = '',
			['width'] = 1,
			['align'] = 'LEFT'
		},{
			['name'] = '',
			['width'] = 65,
			['align'] = 'LEFT',
			['sort'] = "asc"
		},{
			['name'] = 'Character',
			['width'] = 85,
			['align'] = 'LEFT', 
			["comparesort"] = function (table, rowa, rowb, sortbycol)
				local asc = table.cols[sortbycol].sort == "asc"
				local ka,kb
				if KeystoneRollCall.tableData[rowa][1] == -1 then
					ka = KeystoneRollCall.dummyKeystones[KeystoneRollCall.tableData[rowa][2]]
				else
					ka = KeystoneRollCall.db.global.keystones[KeystoneRollCall.tableData[rowa][1]]
				end
				if KeystoneRollCall.tableData[rowb][1] == -1 then
					kb = KeystoneRollCall.dummyKeystones[KeystoneRollCall.tableData[rowb][2]]
				else
					kb = KeystoneRollCall.db.global.keystones[KeystoneRollCall.tableData[rowb][1]]
				end
				return CompareKeystones("name", asc, ka, kb);
			end,
			['sort'] = "asc"
		},{
			['name'] = 'Class',
			['width'] = 95,
			['align'] = 'LEFT', 
			["comparesort"] = function (table, rowa, rowb, sortbycol)
				local asc = table.cols[sortbycol].sort == "asc"
				local ka,kb
				if KeystoneRollCall.tableData[rowa][1] == -1 then
					ka = KeystoneRollCall.dummyKeystones[KeystoneRollCall.tableData[rowa][2]]
				else
					ka = KeystoneRollCall.db.global.keystones[KeystoneRollCall.tableData[rowa][1]]
				end
				if KeystoneRollCall.tableData[rowb][1] == -1 then
					kb = KeystoneRollCall.dummyKeystones[KeystoneRollCall.tableData[rowb][2]]
				else
					kb = KeystoneRollCall.db.global.keystones[KeystoneRollCall.tableData[rowb][1]]
				end
				return CompareKeystones("class", asc, ka, kb);
			end,
			['sort'] = "asc"
		},{
			['name'] = 'Dungeon',
			['width'] = 160,
			['align'] = 'LEFT',
			["comparesort"] = function (table, rowa, rowb, sortbycol)
				local asc = table.cols[sortbycol].sort == "asc"
				local ka,kb
				if KeystoneRollCall.tableData[rowa][1] == -1 then
					ka = KeystoneRollCall.dummyKeystones[KeystoneRollCall.tableData[rowa][2]]
				else
					ka = KeystoneRollCall.db.global.keystones[KeystoneRollCall.tableData[rowa][1]]
				end
				if KeystoneRollCall.tableData[rowb][1] == -1 then
					kb = KeystoneRollCall.dummyKeystones[KeystoneRollCall.tableData[rowb][2]]
				else
					kb = KeystoneRollCall.db.global.keystones[KeystoneRollCall.tableData[rowb][1]]
				end
				return CompareKeystones("dungeon", asc, ka, kb);
			end,
			['sort'] = "asc"
		},{
			['name'] = 'Level',
			['width'] = 40,
			['align'] = 'LEFT',
			["comparesort"] = function (table, rowa, rowb, sortbycol)
				local asc = table.cols[sortbycol].sort == "asc"
				local ka,kb
				if KeystoneRollCall.tableData[rowa][1] == -1 then
					ka = KeystoneRollCall.dummyKeystones[KeystoneRollCall.tableData[rowa][2]]
				else
					ka = KeystoneRollCall.db.global.keystones[KeystoneRollCall.tableData[rowa][1]]
				end
				if KeystoneRollCall.tableData[rowb][1] == -1 then
					kb = KeystoneRollCall.dummyKeystones[KeystoneRollCall.tableData[rowb][2]]
				else
					kb = KeystoneRollCall.db.global.keystones[KeystoneRollCall.tableData[rowb][1]]
				end
				return CompareKeystones("level", asc, ka, kb);
			end,
			['sort'] = "asc"
		},{
			['name'] = "Best Run (This Week)",
			['width'] = 180,
			['align'] = 'LEFT',
		}
	}
	
	KeystoneRollCall.tableData = {};
	KeystoneRollCall.dummyKeystones = {};
	KeystoneRollCall.scrollTable = ScrollingTable:CreateST(cols,24);
	KeystoneRollCall.scrollTable:SetData(KeystoneRollCall.tableData, true);
	KeystoneRollCall.scrollTable:RegisterEvents({
		['OnClick'] = function(rowFrame, cellFrame, data, cols, row, realrow, column, scrollingTable, btn,...)
			if not KeystoneRollCall.tableData or not KeystoneRollCall.tableData[realrow] then
				return
			end
			if KeystoneRollCall.tableData[realrow][1] == -1 then
				local keystone = KeystoneRollCall.dummyKeystones[KeystoneRollCall.tableData[realrow][2]]

				SendChatMessage("If you get the addon Keystone Roll-Call we can easily view each other's keystones, as well as keystones from across our guilds.","WHISPER",  "Common",keystone.name);
			elseif KeystoneRollCall.db.global.keystones[KeystoneRollCall.tableData[realrow][1]] then
				local keystone = KeystoneRollCall.db.global.keystones[KeystoneRollCall.tableData[realrow][1]];
				if btn == "RightButton" then
					if UnitName("target") then
						SendChatMessage(KeystoneToString(keystone),"WHISPER", "COMMON",  UnitName("target"));
					end
				elseif IsInGroup() then
					SendChatMessage(KeystoneToString(keystone),"RAID" ,  "Common")
				end
			
			end
		end,	
		['OnDoubleClick'] = function(rowFrame, cellFrame, data, cols, row, realrow, column, scrollingTable, btn,...)
			if KeystoneRollCall.tableData and KeystoneRollCall.tableData[realrow] and KeystoneRollCall.tableData[realrow][1] ~= -1  and KeystoneRollCall.db.global.keystones[KeystoneRollCall.tableData[realrow][1]] then
				local keystone = KeystoneRollCall.db.global.keystones[KeystoneRollCall.tableData[realrow][1]];
				SendChatMessage(KeystoneToString(keystone),"GUILD" ,  "Common")
			end
		end,	
		['OnEnter'] = function(rowFrame, cellFrame, data, cols, row, realrow, column, scrollingTable,...)
			if not realrow or not KeystoneRollCall.tableData[realrow] then
				return
				
			end
			if KeystoneRollCall.tableData[realrow][1] == -1 then
				KeystoneRollCall.frame:SetStatusText("Click: Suggest Addon")
			else
				KeystoneRollCall.frame:SetStatusText("Left Click: Send to Party   -   Double Click: Send to Guild   -   Right Click: Send to Target")
			end
		end,
		['OnLeave'] = function(rowFrame, cellFrame, data, cols, row, realrow, column, scrollingTable,...)
			KeystoneRollCall.frame:SetStatusText("")
		end,
	});
	
	local tableWrapper = AceGUI:Create('lib-st'):WrapST(KeystoneRollCall.scrollTable);
	tableWrapper.head_offset = 15;
	container:AddChild(tableWrapper);
end

-- Callback function for OnGroupSelected
local function SelectGroup(container, event, group)
	container:ReleaseChildren()
	if group == "personal" then
		DrawPersonalTab(container)
		KeystoneRollCall:ScheduleTimer("PopulatePersonal", .1)
	elseif group == "party" then
		DrawPartyTab(container)
		KeystoneRollCall:ScheduleTimer("PopulateParty", .1)
	elseif group == "guild_online" then   
		DrawGuildOnlineTab(container)
		KeystoneRollCall:ScheduleTimer("PopulateGuildOnline", .1)
	elseif group == "guild_all" then
		DrawGuildAllTab(container)
		KeystoneRollCall:ScheduleTimer("PopulateGuildAll", .1)
	end
end


function KeystoneRollCall:Show()
	KeystoneRollCall:UpdatePlayerKeystone()
	KeystoneRollCall:SendKeystone(1,playerName,"RAID")
	KeystoneRollCall:SendKeystone(1,playerName,"GUILD")
	local frame = AceGUI:Create("Frame")
	_G["KeystoneRollCall_Display"] = frame
	tinsert(UISpecialFrames, "KeystoneRollCall_Display") 
	KeystoneRollCall.frame = frame
	frame:SetTitle("Keystone Roll-Call")
	frame:SetStatusText("")
	frame:EnableResize(false);
	frame:SetCallback("OnClose", function(widget) AceGUI:Release(widget) end)
	-- Fill Layout - the TabGroup widget will fill the whole frame
	frame:SetLayout("Fill")

	local tab =  AceGUI:Create("TabGroup")
	tab:SetLayout("Flow")
	tab:SetTabs({{text="Personal", value="personal"}, {text="Party/Raid", value="party"}, {text="Guild (Online)", value="guild_online"}, {text="Guild (All)", value="guild_all"}})
	tab:SetCallback("OnGroupSelected", SelectGroup)
	tab:SelectTab("personal")

	frame:AddChild(tab)
	tinsert(UISpecialFrames, "KeystoneRollCall_Display") 
end